import QtQuick 1.1
import com.meego 1.0
import "../Components" as Components
import "../Components/constants.js" as UI

Page {
    id: wrapper
    property string pageName: "AlbumPage"
    property int longTappedIndex
    property string currentTab: "tracks"
    property string albumName: albumPageController.AlbumName

    orientationLock: 1 // PageOrientation.LockPortrait

    onAlbumNameChanged: {
        viewTitlePanel.headerText = albumName
    }

    onStatusChanged: {
        switch(status) {
            case PageStatus.Activating: {
                viewTitlePanel.headerText = albumName
                albumPageController.setVisible(true)
            }
            case PageStatus.Deactivating: {
                albumPageController.setVisible(false)
            }
        }
    }

    Components.ContextMenuData {
        id: longTapMenuData
        model: ListModel {
            ListElement {
                action: "Queue"
                subText: "Add this track to Queue"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Add to Playlist"
                subText: "Save this track to Playlist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "View Artist"
                subText: "More from this artist"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share Track"
                subText: "Share this track with others"
                hasCheckBox: false
                actionOn: false
            }
            ListElement {
                action: "Share to Spotify People"
                subText: "Send to a friend's Inbox"
                hasCheckBox: false
                actionOn: false
            }
        }
        onItemActivated: {
            switch(model.get(index).action)
            {
                case "Queue": {
                    albumPageController.handleAddToQueue(longTappedIndex)
                }
                case "Add to Playlist": {
                    rootWindow.showDialog("Save to playlist", saveToPlaylistContent)
                }
                case "View Artist": {
                    artistPageController.clearData()
                    goToPage("ArtistPage")
                    albumPageController.getArtistData(longTappedIndex)
                }
                case "Share Track": {
                    albumPageController.shareLongTappedTrack(longTappedIndex, false)
                }
                case "Share to Spotify People": {
                    albumPageController.shareLongTappedTrack(longTappedIndex, true)
                    goToPage("SpotifySharePage")
                }
            }
        }
    }

    Components.LoadingSpinner {
        z: 1
        invertedSpinner: true
        visible: albumPageController.LoadingAlbumData
        anchors.fill:  parent
    }

    Components.ViewTitlePanel {
        id: viewTitlePanel
        z: 2
        objectName: "viewTitlePanel"
        anchors.top: parent.top
        anchors.topMargin: rootWindow.statusBarHeight
        height: 72
        headerText: "AlbumPage"
        onlineMode: engineLoggedIn
        imageSource: nowPlayingPageController.HasValidTrack ?
                "/usr/share/spotify/pics/Spotify-31.png" : ""
        imageSourcePressed: nowPlayingPageController.HasValidTrack ?
                        "/usr/share/spotify/pics/Spotify-61.png" : ""
        onClicked: {
            if (nowPlayingPageController.HasValidTrack) {
                rootWindow.showNPSheet()
            }
        }
    }

    Rectangle {
        id: headerArea
        color: UI.APP_BACKGROUND_COLOR
        height: 160
        width: parent.width
        anchors.top: viewTitlePanel.bottom
        anchors.leftMargin: 12
        Image {
            id: albumArt
            source: albumPageController.ImagePath
            width: 140
            height: 140
            anchors.left: headerArea.left
            anchors.leftMargin: 10
            anchors.verticalCenter: headerArea.verticalCenter
            asynchronous: true
        }

        Components.TitlePanel {
            id: artistName
            headerText: albumPageController.ArtistName
            anchors.left: albumArt.right
            anchors.right: parent.right
            anchors.top: albumArt.top
            anchors.leftMargin: 15
        }

         Components.PropertiesDetailPanel{
            id: year
            headerText: albumPageController.Year
            anchors.left: albumArt.right
            anchors.top: artistName.bottom
        }

        Components.PropertiesDetailPanel {
            objectName: "trackCount"
            headerText: trackList.count + " tracks, " + albumBrowseModel.duration
            anchors.left: albumArt.right
            anchors.top: year.bottom
        }
        Components.Button {
            id: favouriteButton
            objectName: "favouriteButton"
            width: 49
            height: width
            imageSize: 40
            anchors.right: parent.right
            anchors.bottom: parent.bottom
            anchors.margins: 15
            imagePath: albumPageController.Starred ? "/usr/share/spotify/pics/Spotify-19.png" :
                                                      "/usr/share/spotify/pics/Spotify-23.png"
            onClicked: albumPageController.changeStarredForAll()
        }
        Rectangle {
            id: line
            height: albumPageController.Review != "" ? 0 : 1
            color: "grey"
            anchors.bottom: parent.bottom
            anchors.left: parent.left
            anchors.leftMargin: 15
            anchors.right: parent.right
            anchors.rightMargin: 15
        }
    }

    Item {
        id: tabs
        visible: albumPageController.Review != ""
        height: 50
        width: parent.width
        anchors.top: headerArea.bottom

        Image {
            id: arrow
            anchors.verticalCenter: tabs.top
            anchors.horizontalCenter: if (currentTab == "tracks") {
                return tracks.horizontalCenter
            } else if (currentTab == "review") {
                return review.horizontalCenter
            }
            source: "/usr/share/spotify/pics/icon-m-framework-viewmenu-arrow.svg"
            asynchronous: true
        }

        Rectangle {
            id: tracks
            width: parent.width / 2
            height: parent.height
            anchors.left: parent.left
            anchors.top: tabs.top
            color: currentTab == "tracks" ? "#4f4f4f" : "#232323"
            Text {
                id: tracksText
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 10
                anchors.horizontalCenter: parent.horizontalCenter
                text: "Tracks"
                color: currentTab == "tracks" ? "white" : "grey"
                font.pixelSize: UI.FONT_SIZE_NORMAL
                font.family: UI.FONT_FAMILY
            }
            Rectangle {
                width: 1
                height: 25
                anchors.right: parent.right
                anchors.bottom: tracksText.bottom
                color: "grey"
            }
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    currentTab = "tracks"
                }
            }
        }
        Rectangle {
            id: review
            width: parent.width / 2
            height: parent.height
            anchors.right: parent.right
            anchors.top: tabs.top
            color: currentTab == "review" ? "#4f4f4f" : "#232323"
            Text {
                id: reviewText
                anchors.bottom: parent.bottom
                anchors.bottomMargin: 10
                anchors.horizontalCenter: parent.horizontalCenter
                text: "Review"
                color: currentTab == "review" ? "white" : "grey"
                font.pixelSize: UI.FONT_SIZE_NORMAL
                font.family: UI.FONT_FAMILY
            }
            MouseArea {
                anchors.fill: parent
                onClicked: {
                    currentTab = "review"
                }
            }
        }
    }
    Rectangle {
        width: parent.width
        visible: tabs.visible
        height: 2
        anchors.left: parent.left
        anchors.bottom: tabs.bottom
        color: "grey"
    }

    Item {
        id: albumDataArea
        width: parent.width
        anchors.top: tabs.visible ? tabs.bottom : headerArea.bottom
        anchors.bottom: parent.bottom
        anchors.bottomMargin: albumAndArtistPageToolBar.height

        Components.ScrollBar {
            z: 2
            scrollArea: trackList
        }

        ListView {
            id: trackList
            objectName: "trackList"
            visible: currentTab == "tracks"
            anchors.fill: parent
            model: albumBrowseModel
            clip: true
            cacheBuffer: 800

            delegate: Components.ListItem {
                objectName: "AlbumBrowseListItem"
                labelText1: TrackTitle
                labelText2: albumPageController.ArtistName == "Various Artists" ?
                        TrackArtist + " | " + TrackDurationInString :
                        TrackDurationInString
                starred: TrackStarred
                listReady: TrackAvailable
                onStarClicked: albumPageController.setStarred(index)
                enableAddToQueue: true
                onClicked: {
                    if (albumPageController.listClicked(index)) {
                        goToPage("NowPlayingPage")
                    } else {
                        notifyController.showMessage("Track is not available")
                    }
                }
                onPressAndHold: {
                    viewMenu.menuData = longTapMenuData
                    longTappedIndex = index;
                    viewMenu.open(labelText1, TrackArtist, albumPageController.ImagePath)
                }
                onSwiped: albumPageController.handleAddToQueue(index)
            }
        }

        Components.ScrollBar {
            z: 2
            scrollArea: reviewArea
            allowFastScrolling: false
        }

        Flickable {
            id: reviewArea
            visible: currentTab == "review"
            anchors.fill: parent
            contentHeight: bgText.height + 20
            maximumFlickVelocity: 800
            flickableDirection: Flickable.VerticalFlick
            clip: true

            Text {
                id: bgText
                text: albumPageController.Review
                wrapMode: Text.WordWrap
                textFormat: Text.RichText
                anchors.left: parent.left
                anchors.leftMargin: 15
                anchors.right: parent.right
                anchors.rightMargin: 15
                anchors.top: parent.top
                anchors.topMargin: 15
                color: "black"
                font.pixelSize: UI.FONT_SIZE_NORMAL
                font.family: UI.FONT_FAMILY
                onLinkActivated: albumPageController.linkActivated(link)
            }
        }
    }

    Connections {
        target: albumPageController
        onAlbumLinkClicked: {
            albumPageController.resetAlbumPage()
            currentTab = "tracks"
        }
        onArtistLinkClicked: {
            artistPageController.clearData()
            artistPageController.setTab("overview")
            goToPage("ArtistPage")
        }
        onSearchLinkClicked: {
          //  toolbar.activateSearch(search)
        }
    }

    Component {
        id: saveToPlaylistContent
        Item {
            height: childrenRect.height
            width: dialogLoader.status ==  Loader.Ready ? dialogLoader.item.contentWidth : 0

            ListView {
                id: playlists
                objectName: "playlists"
                clip: true
                width: parent.width
                height: Math.max(150, Math.min(600, count * 80 + 80))
                anchors.bottomMargin: albumAndArtistPageToolBar.height
                model: editablePlaylistsModel
                header: headerComponent
                delegate: Components.ListItem {
                    inverted: true
                    labelText1: playlistName
                    showIcon: false
                    onClicked: {
                        albumPageController.saveToPlaylist(index, longTappedIndex)
                        rootWindow.hideDialog()
                    }
                }

                Component {
                    id: headerComponent
                    Components.ListItem {
                        labelText1: "<Add as a new playlist>"
                        labelText2: albumPageController.longTappedItem(longTappedIndex)
                        showIcon: false
                        onClicked: {
                            albumPageController.saveToPlaylist(-1, longTappedIndex)
                            rootWindow.hideDialog()
                        }
                    }
                }
            }
        }
    }
}
